# -*- coding: utf-8 -*-
# Copyright (C) 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from . import Gmpv9TestCase
from ..gmpv7.testcmds import *  # pylint: disable=unused-wildcard-import, wildcard-import


class Gmpv9AuthenticateTestCase(GmpAuthenticateTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneAgentTestCase(GmpCloneAgentTestCase, Gmpv9TestCase):

    pass


class Gmpv9CloneAlertTestCase(GmpCloneAlertTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneConfigTestCase(GmpCloneConfigTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneReportFormatTestCase(
    GmpCloneReportFormatTestCase, Gmpv9TestCase
):
    pass


class Gmpv9CloneCredentialTestCase(GmpCloneCredentialTestCase, Gmpv9TestCase):

    pass


class Gmpv9HelpTestCase(GmpHelpTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneFilterTestCase(GmpCloneFilterTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneGroupTestCase(GmpCloneGroupTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneNoteTestCase(GmpCloneNoteTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneOverrideTestCase(GmpCloneOverrideTestCase, Gmpv9TestCase):
    pass


class Gmpv9ClonePermissionTestCase(GmpClonePermissionTestCase, Gmpv9TestCase):
    pass


class Gmpv9ClonePortListTestCase(GmpClonePortListTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneRoleTestCase(GmpCloneRoleTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneScannerTestCase(GmpCloneScannerTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneScheduleTestCase(GmpCloneScheduleTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneTagTestCase(GmpCloneTagTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneTargetCommandTestCase(
    GmpCloneTargetCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9CloneTaskTestCase(GmpCloneTaskTestCase, Gmpv9TestCase):
    pass


class Gmpv9CloneUserTestCase(GmpCloneUserTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateAgentTestCase(GmpCreateAgentTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateContainerTaskCommandTestCase(
    GMPCreateContainerTaskCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9CreateGroupTestCase(GmpCreateGroupTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateHostTestCase(GmpCreateHostTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateNoteTestCase(GmpCreateNoteTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateOverrideTestCase(GmpCreateOverrideTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreatePortListTestCase(GmpCreatePortListTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreatePortRangeTestCase(GmpCreatePortRangeTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateRoleTestCase(GmpCreateRoleTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateUserTestCase(GmpCreateUserTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteAgentTestCase(GmpDeleteAgentTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteAlertTestCase(GmpDeleteAlertTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteAssetTestCase(GmpDeleteAssetTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteCredentialTestCase(GmpDeleteCredentialTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteFilterTestCase(GmpDeleteFilterTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteGroupTestCase(GmpDeleteGroupTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteNoteTestCase(GmpDeleteNoteTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteOverrideTestCase(GmpDeleteOverrideTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeletePermissionTestCase(GmpDeletePermissionTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeletePortListTestCase(GmpDeletePortListTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeletePortRangeTestCase(GmpDeletePortRangeTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteReportTestCase(GmpDeleteReportTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteReportFormatTestCase(
    GmpDeleteReportFormatTestCase, Gmpv9TestCase
):
    pass


class Gmpv9DeleteRoleTestCase(GmpDeleteRoleTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteScannerTestCase(GmpDeleteScannerTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteScheduleTestCase(GmpDeleteScheduleTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteTagTestCase(GmpDeleteTagTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteTargetTestCase(GmpDeleteTargetTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteTaskTestCase(GmpDeleteTaskTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteUserTestCase(GmpDeleteUserTestCase, Gmpv9TestCase):
    pass


class Gmpv9DescribeAuthCommandTestCase(
    GmpDescribeAuthCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9EmptyTrashcanCommandTestCase(
    GmpEmptyTrashcanCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9GetAgentTestCase(GmpGetAgentTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetAgentsTestCase(GmpGetAgentsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetAlertTestCase(GmpGetAlertTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetAlertsTestCase(GmpGetAlertsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetAssetTestCase(GmpGetAssetTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetAssetsTestCase(GmpGetAssetsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetCredentialTestCase(GmpGetCredentialTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetCredentialsTestCase(GmpGetCredentialsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetFeedTestCase(GmpGetFeedTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetFeedsTestCase(GmpGetFeedsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetFilterTestCase(GmpGetFilterTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetFiltersTestCase(GmpGetFiltersTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetGroupTestCase(GmpGetGroupTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetGroupsTestCase(GmpGetGroupsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetInfoTestCase(GmpGetInfoTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetInfoListTestCase(GmpGetInfoListTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetNoteTestCase(GmpGetNoteTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetNotesTestCase(GmpGetNotesTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetNvtTestCase(GmpGetNvtTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetNvtFamiliesTestCase(GmpGetNvtFamiliesTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetNvtsTestCase(GmpGetNvtsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetOverrideTestCase(GmpGetOverrideTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetOverridesTestCase(GmpGetOverridesTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetPermissionTestCase(GmpGetPermissionTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetPermissionsTestCase(GmpGetPermissionsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetPortListTestCase(GmpGetPortListTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetPortListsTestCase(GmpGetPortListsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetPreferenceTestCase(GmpGetPreferenceTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetPreferencesTestCase(GmpGetPreferencesTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetReportTestCase(GmpGetReportTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetReportFormatTestCase(GmpGetReportFormatTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetReportFormatsTestCase(GmpGetReportFormatsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetReportsTestCase(GmpGetReportsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetResultTestCase(GmpGetResultTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetResultsTestCase(GmpGetResultsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetRoleTestCase(GmpGetRoleTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetRolesTestCase(GmpGetRolesTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetScannerTestCase(GmpGetScannerTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetScannersTestCase(GmpGetScannersTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetScheduleTestCase(GmpGetScheduleTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetSchedulesTestCase(GmpGetSchedulesTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetSettingTestCase(GmpGetSettingTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetSettingsTestCase(GmpGetSettingsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetSystemReportsTestCase(GmpGetSystemReportsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetTagTestCase(GmpGetTagTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetTagsTestCase(GmpGetTagsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetTargetTestCase(GmpGetTargetTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetTargetsTestCase(GmpGetTargetsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetUserTestCase(GmpGetUserTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetUsersTestCase(GmpGetUsersTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetVersionCommandTestCase(
    GmpGetVersionCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ImportConfigTestCase(GmpImportConfigTestCase, Gmpv9TestCase):
    pass


class Gmpv9ImportReportFormatTestCase(
    GmpImportReportFormatTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ImportReportTestCase(GmpImportReportTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyAgentTestCase(GmpModifyAgentTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyAssetTestCase(GmpModifyAssetTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyAuthTestCase(GmpModifyAuthTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyConfigTestCase(GmpModifyConfigTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyConfigSetCommentTestCase(
    GmpModifyConfigSetCommentTestCase, Gmpv9TestCase
):
    pass


class Gmpv7ModifyConfigSetNameTestCase(
    GmpModifyConfigSetNameTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ModifyConfigSetFamilySelectionTestCase(
    GmpModifyConfigSetFamilySelectionTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ModifyConfigSetNvtPreferenceTestCase(
    GmpModifyConfigSetNvtPreferenceTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ModifyConfigSetNvtSelectionTestCase(
    GmpModifyConfigSetNvtSelectionTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ModifyConfigSetScannerPreferenceTestCase(
    GmpModifyConfigSetScannerPreferenceTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ModifyGroupTestCase(GmpModifyGroupTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyNoteTestCase(GmpModifyNoteTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyOverrideTestCase(GmpModifyOverrideTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyPortListTestCase(GmpModifyPortListTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyRoleTestCase(GmpModifyRoleTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifySettingTestCase(GmpModifySettingTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyTargetTestCase(GmpModifyTargetTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyTaskCommandTestCase(
    GmpModifyTaskCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9ModifyUserTestCase(GmpModifyUserTestCase, Gmpv9TestCase):
    pass


class Gmpv9MoveTaskTestCase(GmpMoveTaskTestCase, Gmpv9TestCase):
    pass


class Gmpv9RestoreTestCase(GmpRestoreTestCase, Gmpv9TestCase):
    pass


class Gmpv9ResumeTaskTestCase(GmpResumeTaskTestCase, Gmpv9TestCase):
    pass


class Gmpv9StartTaskTestCase(GmpStartTaskTestCase, Gmpv9TestCase):
    pass


class Gmpv9StopTaskTestCase(GmpStopTaskTestCase, Gmpv9TestCase):
    pass


class Gmpv9SyncCertCommandTestCase(GmpSyncCertCommandTestCase, Gmpv9TestCase):
    pass


class Gmpv9SyncConfigCommandTestCase(
    GmpSyncConfigCommandTestCase, Gmpv9TestCase
):

    pass


class Gmpv9SyncFeedCommandTestCase(GmpSyncFeedCommandTestCase, Gmpv9TestCase):
    pass


class Gmpv9SyncScapCommandTestCase(GmpSyncScapCommandTestCase, Gmpv9TestCase):
    pass


class Gmpv9TestAlertTestCase(GmpTestAlertTestCase, Gmpv9TestCase):
    pass


class Gmpv9TriggerAlertTestCase(GmpTriggerAlertTestCase, Gmpv9TestCase):
    pass


class Gmpv9VerifyAgentTestCase(GmpVerifyAgentTestCase, Gmpv9TestCase):
    pass


class Gmpv9VerifyReportFormatTestCase(
    GmpVerifyReportFormatTestCase, Gmpv9TestCase
):
    pass


class Gmpv9v7VerifyScannerTestCase(GmpVerifyScannerTestCase, Gmpv9TestCase):
    pass


class Gmpv9v7WithStatementTestCase(GmpWithStatementTestCase, Gmpv9TestCase):
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2020 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=arguments-differ, redefined-builtin, too-many-lines

"""
Module for communication with gvmd in
`Greenbone Management Protocol version 21.04`_

.. _Greenbone Management Protocol version 21.04:
    https://docs.greenbone.net/API/GMP/gmp-21.04.html
"""

from typing import Any, List, Optional, Callable
import numbers

from gvm.utils import deprecation
from gvm.xml import XmlCommand
from gvm.protocols.gmpv7.gmpv7 import _to_comma_list

from gvm.connections import GvmConnection
from gvm.errors import RequiredArgument

from gvm.protocols.base import GvmProtocol

from . import types
from .types import *  # pylint: disable=unused-wildcard-import, wildcard-import

_EMPTY_POLICY_ID = '085569ce-73ed-11df-83c3-002264764cea'

PROTOCOL_VERSION = (21, 4)

Severity = numbers.Real


class GmpV214Mixin(GvmProtocol):
    types = types

    def __init__(
        self,
        connection: GvmConnection,
        *,
        transform: Optional[Callable[[str], Any]] = None
    ):
        super().__init__(connection, transform=transform)

        # Is authenticated on gvmd
        self._authenticated = False

    def create_note(
        self,
        text: str,
        nvt_oid: str,
        *,
        days_active: Optional[int] = None,
        hosts: Optional[List[str]] = None,
        port: Optional[int] = None,
        result_id: Optional[str] = None,
        severity: Optional[Severity] = None,
        task_id: Optional[str] = None,
        threat: Optional[SeverityLevel] = None
    ) -> Any:
        """Create a new note

        Arguments:
            text: Text of the new note
            nvt_id: OID of the nvt to which note applies
            days_active: Days note will be active. -1 on
                always, 0 off
            hosts: A list of hosts addresses
            port: Port to which the note applies
            result_id: UUID of a result to which note applies
            severity: Severity to which note applies
            task_id: UUID of task to which note applies
            threat: Severity level to which note applies. Will be converted to
                severity.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not text:
            raise RequiredArgument(
                function=self.create_note.__name__, argument='text'
            )

        if not nvt_oid:
            raise RequiredArgument(
                function=self.create_note.__name__, argument='nvt_oid'
            )

        cmd = XmlCommand("create_note")
        cmd.add_element("text", text)
        cmd.add_element("nvt", attrs={"oid": nvt_oid})

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", _to_comma_list(hosts))

        if port:
            cmd.add_element("port", str(port))

        if result_id:
            cmd.add_element("result", attrs={"id": result_id})

        if severity:
            cmd.add_element("severity", str(severity))

        if task_id:
            cmd.add_element("task", attrs={"id": task_id})

        if threat is not None:
            deprecation(
                "The threat parameter has been removed in GMP"
                " version {}{}".format(
                    self.get_protocol_version()[0],
                    self.get_protocol_version()[1],
                )
            )

        return self._send_xml_command(cmd)

    def create_override(
        self,
        text: str,
        nvt_oid: str,
        *,
        days_active: Optional[int] = None,
        hosts: Optional[List[str]] = None,
        port: Optional[int] = None,
        result_id: Optional[str] = None,
        severity: Optional[Severity] = None,
        new_severity: Optional[Severity] = None,
        task_id: Optional[str] = None,
        threat: Optional[SeverityLevel] = None,
        new_threat: Optional[SeverityLevel] = None
    ) -> Any:
        """Create a new override

        Arguments:
            text: Text of the new override
            nvt_id: OID of the nvt to which override applies
            days_active: Days override will be active. -1 on always, 0 off
            hosts: A list of host addresses
            port: Port to which the override applies
            result_id: UUID of a result to which override applies
            severity: Severity to which override applies
            new_severity: New severity for result
            task_id: UUID of task to which override applies
            threat: Severity level to which override applies. Will be converted
                to severity.
            new_threat: New severity level for results. Will be converted to
                new_severity.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not text:
            raise RequiredArgument(
                function=self.create_override.__name__, argument='text'
            )

        if not nvt_oid:
            raise RequiredArgument(
                function=self.create_override.__name__, argument='nvt_oid'
            )

        cmd = XmlCommand("create_override")
        cmd.add_element("text", text)
        cmd.add_element("nvt", attrs={"oid": nvt_oid})

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", _to_comma_list(hosts))

        if port:
            cmd.add_element("port", str(port))

        if result_id:
            cmd.add_element("result", attrs={"id": result_id})

        if severity:
            cmd.add_element("severity", str(severity))

        if new_severity:
            cmd.add_element("new_severity", str(new_severity))

        if task_id:
            cmd.add_element("task", attrs={"id": task_id})

        if threat is not None:
            deprecation(
                "The threat parameter has been removed in GMP"
                " version {}{}".format(
                    self.get_protocol_version()[0],
                    self.get_protocol_version()[1],
                )
            )

        if new_threat is not None:
            deprecation(
                "The new_threat parameter has been removed in GMP"
                " version {}{}".format(
                    self.get_protocol_version()[0],
                    self.get_protocol_version()[1],
                )
            )

        return self._send_xml_command(cmd)

    def modify_note(
        self,
        note_id: str,
        text: str,
        *,
        days_active: Optional[int] = None,
        hosts: Optional[List[str]] = None,
        port: Optional[int] = None,
        result_id: Optional[str] = None,
        severity: Optional[Severity] = None,
        task_id: Optional[str] = None,
        threat: Optional[SeverityLevel] = None
    ) -> Any:
        """Modifies an existing note.

        Arguments:
            note_id: UUID of note to modify.
            text: The text of the note.
            days_active: Days note will be active. -1 on always, 0 off.
            hosts: A list of hosts addresses
            port: Port to which note applies.
            result_id: Result to which note applies.
            severity: Severity to which note applies.
            task_id: Task to which note applies.
            threat: Threat level to which note applies. Will be converted to
                severity.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not note_id:
            raise RequiredArgument(
                function=self.modify_note.__name__, argument='note_id'
            )

        if not text:
            raise RequiredArgument(
                function=self.modify_note.__name__, argument='text'
            )

        cmd = XmlCommand("modify_note")
        cmd.set_attribute("note_id", note_id)
        cmd.add_element("text", text)

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", _to_comma_list(hosts))

        if port:
            cmd.add_element("port", str(port))

        if result_id:
            cmd.add_element("result", attrs={"id": result_id})

        if severity:
            cmd.add_element("severity", str(severity))

        if task_id:
            cmd.add_element("task", attrs={"id": task_id})

        if threat is not None:
            deprecation(
                "The threat parameter has been removed in GMP"
                " version {}{}".format(
                    self.get_protocol_version()[0],
                    self.get_protocol_version()[1],
                )
            )

        return self._send_xml_command(cmd)

    def modify_override(
        self,
        override_id: str,
        text: str,
        *,
        days_active: Optional[int] = None,
        hosts: Optional[List[str]] = None,
        port: Optional[int] = None,
        result_id: Optional[str] = None,
        severity: Optional[Severity] = None,
        new_severity: Optional[Severity] = None,
        task_id: Optional[str] = None,
        threat: Optional[SeverityLevel] = None,
        new_threat: Optional[SeverityLevel] = None
    ) -> Any:
        """Modifies an existing override.

        Arguments:
            override_id: UUID of override to modify.
            text: The text of the override.
            days_active: Days override will be active. -1 on always,
                0 off.
            hosts: A list of host addresses
            port: Port to which override applies.
            result_id: Result to which override applies.
            severity: Severity to which override applies.
            new_severity: New severity score for result.
            task_id: Task to which override applies.
            threat: Threat level to which override applies.
                Will be converted to severity.
            new_threat: New threat level for results. Will be converted to
                new_severity.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not override_id:
            raise RequiredArgument(
                function=self.modify_override.__name__, argument='override_id'
            )
        if not text:
            raise RequiredArgument(
                function=self.modify_override.__name__, argument='text'
            )

        cmd = XmlCommand("modify_override")
        cmd.set_attribute("override_id", override_id)
        cmd.add_element("text", text)

        if days_active is not None:
            cmd.add_element("active", str(days_active))

        if hosts:
            cmd.add_element("hosts", _to_comma_list(hosts))

        if port:
            cmd.add_element("port", str(port))

        if result_id:
            cmd.add_element("result", attrs={"id": result_id})

        if severity:
            cmd.add_element("severity", str(severity))

        if new_severity:
            cmd.add_element("new_severity", str(new_severity))

        if task_id:
            cmd.add_element("task", attrs={"id": task_id})

        if threat is not None:
            deprecation(
                "The threat parameter has been removed in GMP"
                " version {}{}".format(
                    self.get_protocol_version()[0],
                    self.get_protocol_version()[1],
                )
            )

        if new_threat is not None:
            deprecation(
                "The new_threat parameter has been removed in GMP"
                " version {}{}".format(
                    self.get_protocol_version()[0],
                    self.get_protocol_version()[1],
                )
            )

        return self._send_xml_command(cmd)



# -*- coding: utf-8 -*-
# Copyright (C) 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from . import Gmpv208TestCase
from ..gmpv7.testcmds import *  # pylint: disable=unused-wildcard-import,wildcard-import


class Gmpv208AuthenticateTestCase(GmpAuthenticateTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneAlertTestCase(GmpCloneAlertTestCase, Gmpv208TestCase):

    pass


class Gmpv208CloneConfigTestCase(GmpCloneConfigTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneReportFormatTestCase(
    GmpCloneReportFormatTestCase, Gmpv208TestCase
):
    pass


class Gmpv208CloneCredentialTestCase(
    GmpCloneCredentialTestCase, Gmpv208TestCase
):
    pass


class Gmpv208CloneFilterTestCase(GmpCloneFilterTestCase, Gmpv208TestCase):
    pass


class Gmpv208HelpTestCase(GmpHelpTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneGroupTestCase(GmpCloneGroupTestCase, Gmpv208TestCase):

    pass


class Gmpv208CloneNoteTestCase(GmpCloneNoteTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneOverrideTestCase(GmpCloneOverrideTestCase, Gmpv208TestCase):
    pass


class Gmpv208ClonePermissionTestCase(
    GmpClonePermissionTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ClonePortListTestCase(GmpClonePortListTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneRoleTestCase(GmpCloneRoleTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneScannerTestCase(GmpCloneScannerTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneScheduleTestCase(GmpCloneScheduleTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneTagTestCase(GmpCloneTagTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneTargetCommandTestCase(
    GmpCloneTargetCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208CloneTaskTestCase(GmpCloneTaskTestCase, Gmpv208TestCase):
    pass


class Gmpv208CloneUserTestCase(GmpCloneUserTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreateContainerTaskCommandTestCase(
    GMPCreateContainerTaskCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208CreateGroupTestCase(GmpCreateGroupTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreateHostTestCase(GmpCreateHostTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreateNoteTestCase(GmpCreateNoteTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreateOverrideTestCase(GmpCreateOverrideTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreatePortListTestCase(GmpCreatePortListTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreatePortRangeTestCase(
    GmpCreatePortRangeTestCase, Gmpv208TestCase
):
    pass


class Gmpv208CreateRoleTestCase(GmpCreateRoleTestCase, Gmpv208TestCase):
    pass


class Gmpv208CreateUserTestCase(GmpCreateUserTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteAlertTestCase(GmpDeleteAlertTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteAssetTestCase(GmpDeleteAssetTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteCredentialTestCase(
    GmpDeleteCredentialTestCase, Gmpv208TestCase
):
    pass


class Gmpv208DeleteFilterTestCase(GmpDeleteFilterTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteGroupTestCase(GmpDeleteGroupTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteNoteTestCase(GmpDeleteNoteTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteOverrideTestCase(GmpDeleteOverrideTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeletePermissionTestCase(
    GmpDeletePermissionTestCase, Gmpv208TestCase
):
    pass


class Gmpv208DeletePortListTestCase(GmpDeletePortListTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeletePortRangeTestCase(
    GmpDeletePortRangeTestCase, Gmpv208TestCase
):
    pass


class Gmpv208DeleteReportTestCase(GmpDeleteReportTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteReportFormatTestCase(
    GmpDeleteReportFormatTestCase, Gmpv208TestCase
):
    pass


class Gmpv208DeleteRoleTestCase(GmpDeleteRoleTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteScannerTestCase(GmpDeleteScannerTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteScheduleTestCase(GmpDeleteScheduleTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteTagTestCase(GmpDeleteTagTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteTargetTestCase(GmpDeleteTargetTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteTaskTestCase(GmpDeleteTaskTestCase, Gmpv208TestCase):
    pass


class Gmpv208DeleteUserTestCase(GmpDeleteUserTestCase, Gmpv208TestCase):
    pass


class Gmpv208DescribeAuthCommandTestCase(
    GmpDescribeAuthCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208EmptyTrashcanCommandTestCase(
    GmpEmptyTrashcanCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208GetAlertTestCase(GmpGetAlertTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetAlertsTestCase(GmpGetAlertsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetAssetTestCase(GmpGetAssetTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetAssetsTestCase(GmpGetAssetsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetCredentialTestCase(GmpGetCredentialTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetCredentialsTestCase(GmpGetCredentialsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetFeedTestCase(GmpGetFeedTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetFeedsTestCase(GmpGetFeedsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetFilterTestCase(GmpGetFilterTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetFiltersTestCase(GmpGetFiltersTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetGroupTestCase(GmpGetGroupTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetGroupsTestCase(GmpGetGroupsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetInfoListTestCase(GmpGetInfoListTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetNoteTestCase(GmpGetNoteTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetNotesTestCase(GmpGetNotesTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetNvtTestCase(GmpGetNvtTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetNvtFamiliesTestCase(GmpGetNvtFamiliesTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetNvtsTestCase(GmpGetNvtsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetOverrideTestCase(GmpGetOverrideTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetOverridesTestCase(GmpGetOverridesTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetPermissionTestCase(GmpGetPermissionTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetPermissionsTestCase(GmpGetPermissionsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetPortListTestCase(GmpGetPortListTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetPortListsTestCase(GmpGetPortListsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetPreferenceTestCase(GmpGetPreferenceTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetPreferencesTestCase(GmpGetPreferencesTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetReportTestCase(GmpGetReportTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetReportFormatTestCase(
    GmpGetReportFormatTestCase, Gmpv208TestCase
):
    pass


class Gmpv208GetReportFormatsTestCase(
    GmpGetReportFormatsTestCase, Gmpv208TestCase
):
    pass


class Gmpv208GetReportsTestCase(GmpGetReportsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetResultTestCase(GmpGetResultTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetResultsTestCase(GmpGetResultsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetRoleTestCase(GmpGetRoleTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetRolesTestCase(GmpGetRolesTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetScannerTestCase(GmpGetScannerTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetScannersTestCase(GmpGetScannersTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetScheduleTestCase(GmpGetScheduleTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetSchedulesTestCase(GmpGetSchedulesTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetSettingTestCase(GmpGetSettingTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetSettingsTestCase(GmpGetSettingsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetSystemReportsTestCase(
    GmpGetSystemReportsTestCase, Gmpv208TestCase
):
    pass


class Gmpv208GetTagTestCase(GmpGetTagTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetTagsTestCase(GmpGetTagsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetTargetTestCase(GmpGetTargetTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetTargetsTestCase(GmpGetTargetsTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetUserTestCase(GmpGetUserTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetUsersTestCase(GmpGetUsersTestCase, Gmpv208TestCase):
    pass


class Gmpv208GetVersionCommandTestCase(
    GmpGetVersionCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ImportConfigTestCase(GmpImportConfigTestCase, Gmpv208TestCase):
    pass


class Gmpv208ImportReportFormatTestCase(
    GmpImportReportFormatTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ImportReportTestCase(GmpImportReportTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyAssetTestCase(GmpModifyAssetTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyAuthTestCase(GmpModifyAuthTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyConfigTestCase(GmpModifyConfigTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyConfigSetCommentTestCase(
    GmpModifyConfigSetCommentTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ModifyConfigSetFamilySelectionTestCase(
    GmpModifyConfigSetFamilySelectionTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ModifyConfigSetNvtPreferenceTestCase(
    GmpModifyConfigSetNvtPreferenceTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ModifyConfigSetNvtSelectionTestCase(
    GmpModifyConfigSetNvtSelectionTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ModifyConfigSetScannerPreferenceTestCase(
    GmpModifyConfigSetScannerPreferenceTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ModifyGroupTestCase(GmpModifyGroupTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyNoteTestCase(GmpModifyNoteTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyOverrideTestCase(GmpModifyOverrideTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyPortListTestCase(GmpModifyPortListTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyRoleTestCase(GmpModifyRoleTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifySettingTestCase(GmpModifySettingTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyTargetTestCase(GmpModifyTargetTestCase, Gmpv208TestCase):
    pass


class Gmpv208ModifyTaskCommandTestCase(
    GmpModifyTaskCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208ModifyUserTestCase(GmpModifyUserTestCase, Gmpv208TestCase):
    pass


class Gmpv208MoveTaskTestCase(GmpMoveTaskTestCase, Gmpv208TestCase):
    pass


class Gmpv208RestoreTestCase(GmpRestoreTestCase, Gmpv208TestCase):
    pass


class Gmpv208ResumeTaskTestCase(GmpResumeTaskTestCase, Gmpv208TestCase):
    pass


class Gmpv208StartTaskTestCase(GmpStartTaskTestCase, Gmpv208TestCase):
    pass


class Gmpv208StopTaskTestCase(GmpStopTaskTestCase, Gmpv208TestCase):
    pass


class Gmpv208SyncCertCommandTestCase(
    GmpSyncCertCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208SyncConfigCommandTestCase(
    GmpSyncConfigCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208SyncFeedCommandTestCase(
    GmpSyncFeedCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208SyncScapCommandTestCase(
    GmpSyncScapCommandTestCase, Gmpv208TestCase
):
    pass


class Gmpv208TestAlertTestCase(GmpTestAlertTestCase, Gmpv208TestCase):
    pass


class Gmpv208TriggerAlertTestCase(GmpTriggerAlertTestCase, Gmpv208TestCase):
    pass


class Gmpv208VerifyReportFormatTestCase(
    GmpVerifyReportFormatTestCase, Gmpv208TestCase
):
    pass


class Gmpv208v7VerifyScannerTestCase(GmpVerifyScannerTestCase, Gmpv208TestCase):
    pass


class Gmpv208v7WithStatementTestCase(GmpWithStatementTestCase, Gmpv208TestCase):
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2018 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from gvm.errors import RequiredArgument


class GmpCreatePortListTestCase:
    def test_create_port_list_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name=None, port_range='T:1-1234')

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name='', port_range='T:1-1234')

    def test_create_port_list_missing_port_range(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name='foo', port_range=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_list(name='foo', port_range='')

    def test_create_port_list(self):
        self.gmp.create_port_list(name='foo', port_range='T:1-1234')

        self.connection.send.has_been_called_with(
            '<create_port_list>'
            '<name>foo</name>'
            '<port_range>T:1-1234</port_range>'
            '</create_port_list>'
        )

    def test_create_port_list_with_comment(self):
        self.gmp.create_port_list(
            name='foo', port_range='T:1-1234', comment='lorem'
        )

        self.connection.send.has_been_called_with(
            '<create_port_list>'
            '<name>foo</name>'
            '<port_range>T:1-1234</port_range>'
            '<comment>lorem</comment>'
            '</create_port_list>'
        )


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright (C) 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from enum import Enum

from typing import Optional

from gvm.errors import InvalidArgument

__all__ = [
    "AlertCondition",
    "AlertEvent",
    "AlertMethod",
    "AliveTest",
    "AssetType",
    "CredentialType",
    "CredentialFormat",
    "EntityType",
    "FeedType",
    "FilterType",
    "HostsOrdering",
    "InfoType",
    "PermissionSubjectType",
    "PortRangeType",
    "ReportFormatType",
    "ScannerType",
    "SeverityLevel",
    "SnmpAuthAlgorithm",
    "SnmpPrivacyAlgorithm",
    "TimeUnit",
    "get_alive_test_from_string",
    "get_alert_condition_from_string",
    "get_alert_event_from_string",
    "get_alert_method_from_string",
    "get_asset_type_from_string",
    "get_credential_format_from_string",
    "get_credential_type_from_string",
    "get_entity_type_from_string",
    "get_feed_type_from_string",
    "get_filter_type_from_string",
    "get_hosts_ordering_from_string",
    "get_info_type_from_string",
    "get_permission_subject_type_from_string",
    "get_port_range_type_from_string",
    "get_report_format_id_from_string",
    "get_scanner_type_from_string",
    "get_severity_level_from_string",
    "get_snmp_auth_algorithm_from_string",
    "get_snmp_privacy_algorithm_from_string",
    "get_time_unit_from_string",
]


class AlertEvent(Enum):
    """Enum for alert event types """

    TASK_RUN_STATUS_CHANGED = 'Task run status changed'
    UPDATED_SECINFO_ARRIVED = 'Updated SecInfo arrived'
    NEW_SECINFO_ARRIVED = 'New SecInfo arrived'


def get_alert_event_from_string(
    alert_event: Optional[str],
) -> Optional[AlertEvent]:
    """Convert an alert event string into a AlertEvent instance """
    if not alert_event:
        return None

    alert_event = alert_event.lower()

    if alert_event == 'task run status changed':
        return AlertEvent.TASK_RUN_STATUS_CHANGED

    if alert_event == 'updated secinfo arrived':
        return AlertEvent.UPDATED_SECINFO_ARRIVED

    if alert_event == 'new secinfo arrived':
        return AlertEvent.NEW_SECINFO_ARRIVED

    raise InvalidArgument(
        argument='alert_event', function=get_alert_event_from_string.__name__
    )


class AlertCondition(Enum):
    """Enum for alert condition types """

    ALWAYS = 'Always'
    SEVERITY_AT_LEAST = 'Severity at least'
    FILTER_COUNT_CHANGED = 'Filter count changed'
    FILTER_COUNT_AT_LEAST = 'Filter count at least'


def get_alert_condition_from_string(
    alert_condition: Optional[str],
) -> Optional[AlertCondition]:
    """Convert an alert condition string into a AlertCondition instance """
    if not alert_condition:
        return None

    alert_condition = alert_condition.lower()

    if alert_condition == 'always':
        return AlertCondition.ALWAYS

    if alert_condition == 'filter count changed':
        return AlertCondition.FILTER_COUNT_CHANGED

    if alert_condition == 'filter count at least':
        return AlertCondition.FILTER_COUNT_AT_LEAST

    if alert_condition == 'severity at least':
        return AlertCondition.SEVERITY_AT_LEAST

    raise InvalidArgument(
        argument='alert_condition',
        function=get_alert_condition_from_string.__name__,
    )


class AlertMethod(Enum):
    """Enum for alert method type"""

    SCP = "SCP"
    SEND = "Send"
    SMB = "SMB"
    SNMP = "SNMP"
    SYSLOG = "Syslog"
    EMAIL = "Email"
    START_TASK = "Start Task"
    HTTP_GET = "HTTP Get"
    SOURCEFIRE_CONNECTOR = "Sourcefire Connector"
    VERINICE_CONNECTOR = "verinice Connector"


def get_alert_method_from_string(
    alert_method: Optional[str],
) -> Optional[AlertMethod]:
    """Convert an alert method string into a AlertCondition instance """
    if not alert_method:
        return None

    alert_method = alert_method.upper()

    if alert_method == 'START TASK':
        return AlertMethod.START_TASK

    if alert_method == 'HTTP GET':
        return AlertMethod.HTTP_GET

    if alert_method == 'SOURCEFIRE CONNECTOR':
        return AlertMethod.SOURCEFIRE_CONNECTOR

    if alert_method == 'VERINICE CONNECTOR':
        return AlertMethod.VERINICE_CONNECTOR

    try:
        return AlertMethod[alert_method]
    except KeyError:
        raise InvalidArgument(
            argument='alert_method',
            function=get_alert_method_from_string.__name__,
        ) from None


class AliveTest(Enum):
    """Enum for choosing an alive test """

    SCAN_CONFIG_DEFAULT = 'Scan Config Default'
    ICMP_PING = 'ICMP Ping'
    TCP_ACK_SERVICE_PING = 'TCP-ACK Service Ping'
    TCP_SYN_SERVICE_PING = 'TCP-SYN Service Ping'
    ARP_PING = 'ARP Ping'
    APR_PING = 'ARP Ping'  # Alias for ARP_PING
    ICMP_AND_TCP_ACK_SERVICE_PING = 'ICMP & TCP-ACK Service Ping'
    ICMP_AND_ARP_PING = 'ICMP & ARP Ping'
    TCP_ACK_SERVICE_AND_ARP_PING = 'TCP-ACK Service & ARP Ping'
    ICMP_TCP_ACK_SERVICE_AND_ARP_PING = (  # pylint: disable=invalid-name
        'ICMP, TCP-ACK Service & ARP Ping'
    )
    CONSIDER_ALIVE = 'Consider Alive'


def get_alive_test_from_string(
    alive_test: Optional[str],
) -> Optional[AliveTest]:
    """Convert an alive test string into a AliveTest instance """
    if not alive_test:
        return None

    alive_test = alive_test.lower()

    if alive_test == 'scan config default':
        return AliveTest.SCAN_CONFIG_DEFAULT

    if alive_test == 'icmp ping':
        return AliveTest.ICMP_PING

    if alive_test == 'tcp-ack service ping':
        return AliveTest.TCP_ACK_SERVICE_PING

    if alive_test == 'tcp-syn service ping':
        return AliveTest.TCP_SYN_SERVICE_PING

    if alive_test == 'arp ping':
        return AliveTest.ARP_PING

    if alive_test == 'icmp & tcp-ack service ping':
        return AliveTest.ICMP_AND_TCP_ACK_SERVICE_PING

    if alive_test == 'icmp & arp ping':
        return AliveTest.ICMP_AND_ARP_PING

    if alive_test == 'tcp-ack service & arp ping':
        return AliveTest.TCP_ACK_SERVICE_AND_ARP_PING

    if alive_test == 'icmp, tcp-ack service & arp ping':
        return AliveTest.ICMP_TCP_ACK_SERVICE_AND_ARP_PING

    if alive_test == 'consider alive':
        return AliveTest.CONSIDER_ALIVE

    raise InvalidArgument(
        argument='alive_test', function=get_alive_test_from_string.__name__
    )


class AssetType(Enum):
    """"Enum for asset types """

    OPERATING_SYSTEM = 'os'
    HOST = 'host'


def get_asset_type_from_string(
    asset_type: Optional[str],
) -> Optional[AssetType]:
    if not asset_type:
        return None

    if asset_type == 'os':
        return AssetType.OPERATING_SYSTEM

    try:
        return AssetType[asset_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='asset_type', function=get_asset_type_from_string.__name__
        ) from None


class CredentialFormat(Enum):
    """ Enum for credential format """

    KEY = 'key'
    RPM = 'rpm'
    DEB = 'deb'
    EXE = 'exe'
    PEM = 'pem'


def get_credential_format_from_string(
    credential_format: Optional[str],
) -> Optional[CredentialFormat]:
    if not credential_format:
        return None

    try:
        return CredentialFormat[credential_format.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='credential_format',
            function=get_credential_format_from_string.__name__,
        ) from None


class CredentialType(Enum):
    """Enum for credential types """

    CLIENT_CERTIFICATE = 'cc'
    SNMP = 'snmp'
    USERNAME_PASSWORD = 'up'
    USERNAME_SSH_KEY = 'usk'


def get_credential_type_from_string(
    credential_type: Optional[str],
) -> Optional[CredentialType]:
    """Convert a credential type string into a CredentialType instance"""
    if not credential_type:
        return None

    try:
        return CredentialType[credential_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='credential_type',
            function=get_credential_type_from_string.__name__,
        ) from None


class EntityType(Enum):
    """Enum for entity types """

    AGENT = "agent"
    ALERT = "alert"
    ASSET = "asset"
    CERT_BUND_ADV = "cert_bund_adv"
    CPE = "cpe"
    CREDENTIAL = "credential"
    CVE = "cve"
    DFN_CERT_ADV = "dfn_cert_adv"
    FILTER = "filter"
    GROUP = "group"
    HOST = "host"
    INFO = "info"
    NOTE = "note"
    NVT = "nvt"
    OPERATING_SYSTEM = "os"
    OVALDEF = "ovaldef"
    OVERRIDE = "override"
    PERMISSION = "permission"
    PORT_LIST = "port_list"
    REPORT = "report"
    REPORT_FORMAT = "report_format"
    RESULT = "result"
    ROLE = "role"
    SCAN_CONFIG = "config"
    SCANNER = "scanner"
    SCHEDULE = "schedule"
    TAG = "tag"
    TARGET = "target"
    TASK = "task"
    USER = "user"


def get_entity_type_from_string(
    entity_type: Optional[str],
) -> Optional[EntityType]:
    """Convert a entity type string to an actual EntityType instance

    Arguments:
        entity_type: Resource type string to convert to a EntityType
    """
    if not entity_type:
        return None

    if entity_type == 'config':
        return EntityType.SCAN_CONFIG
    if entity_type == 'os':
        return EntityType.OPERATING_SYSTEM

    try:
        return EntityType[entity_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='entity_type',
            function=get_entity_type_from_string.__name__,
        ) from None


class FeedType(Enum):
    """Enum for feed types """

    NVT = "NVT"
    CERT = "CERT"
    SCAP = "SCAP"


def get_feed_type_from_string(feed_type: Optional[str]) -> Optional[FeedType]:
    """Convert a feed type string into a FeedType instance"""
    if not feed_type:
        return None

    try:
        return FeedType[feed_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='feed_type', function=get_feed_type_from_string.__name__
        ) from None


class FilterType(Enum):
    """Enum for filter types """

    AGENT = "agent"
    ALERT = "alert"
    ASSET = "asset"
    SCAN_CONFIG = "config"
    CREDENTIAL = "credential"
    FILTER = "filter"
    GROUP = "group"
    HOST = "host"
    NOTE = "note"
    OPERATING_SYSTEM = "os"
    OVERRIDE = "override"
    PERMISSION = "permission"
    PORT_LIST = "port_list"
    REPORT = "report"
    REPORT_FORMAT = "report_format"
    RESULT = "result"
    ROLE = "role"
    SCHEDULE = "schedule"
    ALL_SECINFO = "secinfo"
    TAG = "tag"
    TARGET = "target"
    TASK = "task"
    USER = "user"


def get_filter_type_from_string(
    filter_type: Optional[str],
) -> Optional[FilterType]:
    """Convert a filter type string to an actual FilterType instance

    Arguments:
        filter_type: Filter type string to convert to a FilterType
    """
    if not filter_type:
        return None

    if filter_type == 'os':
        return FilterType.OPERATING_SYSTEM

    if filter_type == 'config':
        return FilterType.SCAN_CONFIG

    if filter_type == 'secinfo':
        return FilterType.ALL_SECINFO

    try:
        return FilterType[filter_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='filter_type',
            function=get_filter_type_from_string.__name__,
        ) from None


class HostsOrdering(Enum):
    """ Enum for host ordering during scans """

    SEQUENTIAL = "sequential"
    RANDOM = "random"
    REVERSE = "reverse"


def get_hosts_ordering_from_string(
    hosts_ordering: Optional[str],
) -> Optional[HostsOrdering]:
    """Convert a hosts ordering string to an actual HostsOrdering instance

    Arguments:
        hosts_ordering: Host ordering string to convert to a HostsOrdering
    """
    if not hosts_ordering:
        return None
    try:
        return HostsOrdering[hosts_ordering.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='hosts_ordering',
            function=get_hosts_ordering_from_string.__name__,
        ) from None


class InfoType(Enum):
    """ Enum for info types """

    CERT_BUND_ADV = "CERT_BUND_ADV"
    CPE = "CPE"
    CVE = "CVE"
    DFN_CERT_ADV = "DFN_CERT_ADV"
    OVALDEF = "OVALDEF"
    NVT = "NVT"
    ALLINFO = "ALLINFO"


def get_info_type_from_string(info_type: Optional[str]) -> Optional[InfoType]:
    """Convert a info type string to an actual InfoType instance

    Arguments:
        info_type: Info type string to convert to a InfoType
    """
    if not info_type:
        return None
    try:
        return InfoType[info_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='info_type', function=get_info_type_from_string.__name__
        ) from None


class PermissionSubjectType(Enum):
    """ Enum for permission subject type """

    USER = 'user'
    GROUP = 'group'
    ROLE = 'role'


def get_permission_subject_type_from_string(
    subject_type: Optional[str],
) -> Optional[PermissionSubjectType]:
    """Convert a permission subject type string to an actual
    PermissionSubjectType instance

    Arguments:
        subject_type: Permission subject type string to convert to a
            PermissionSubjectType
    """
    if not subject_type:
        return None

    try:
        return PermissionSubjectType[subject_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='subject_type',
            function=get_permission_subject_type_from_string.__name__,
        ) from None


class PortRangeType(Enum):
    """ Enum for port range type """

    TCP = 'TCP'
    UDP = 'UDP'


def get_port_range_type_from_string(
    port_range_type: Optional[str],
) -> Optional[PortRangeType]:
    """Convert a port range type string to an actual PortRangeType instance

    Arguments:
        port_range_type: Port range type string to convert to a PortRangeType
    """
    if not port_range_type:
        return None

    try:
        return PortRangeType[port_range_type.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='port_range_type',
            function=get_port_range_type_from_string.__name__,
        ) from None


class ReportFormatType(Enum):
    """ Enum for builtin report formats """

    ANONYMOUS_XML = '5057e5cc-b825-11e4-9d0e-28d24461215b'
    ARF = '910200ca-dc05-11e1-954f-406186ea4fc5'
    CPE = '5ceff8ba-1f62-11e1-ab9f-406186ea4fc5'
    CSV_HOSTS = '9087b18c-626c-11e3-8892-406186ea4fc5"'
    CSV_RESULTS = 'c1645568-627a-11e3-a660-406186ea4fc5'
    GCR_PDF = 'dc51a40a-c022-11e9-b02d-3f7ca5bdcb11'
    GSR_HTML = 'ffa123c9-a2d2-409e-bbbb-a6c1385dbeaa'
    GSR_PDF = '35ba7077-dc85-42ef-87c9-b0eda7e903b6'
    GXCR_PDF = 'f0d348de-c022-11e9-bc4c-4bf1d5e1a8ca'
    GXR_PDF = 'ebbc7f34-8ae5-11e1-b07b-001f29eadec8'
    ITG = '77bd6c4a-1f62-11e1-abf0-406186ea4fc5'
    LATEX = 'a684c02c-b531-11e1-bdc2-406186ea4fc5'
    NBE = '9ca6fe72-1f62-11e1-9e7c-406186ea4fc5'
    PDF = 'c402cc3e-b531-11e1-9163-406186ea4fc5'
    SVG = '9e5e5deb-879e-4ecc-8be6-a71cd0875cdd'
    TXT = 'a3810a62-1f62-11e1-9219-406186ea4fc5'
    VERINICE_ISM = 'c15ad349-bd8d-457a-880a-c7056532ee15'
    VERINICE_ITG = '50c9950a-f326-11e4-800c-28d24461215b'
    XML = 'a994b278-1f62-11e1-96ac-406186ea4fc5'


def get_report_format_id_from_string(
    report_format: Optional[str],
) -> Optional[ReportFormatType]:
    """Convert an report format name into a ReportFormatType instance """
    if not report_format:
        return None

    report_format = report_format.lower()

    if report_format == 'anonymous xml':
        return ReportFormatType.ANONYMOUS_XML

    if report_format == 'arf':
        return ReportFormatType.ARF

    if report_format == 'cpe':
        return ReportFormatType.CPE

    if report_format == 'csv hosts':
        return ReportFormatType.CSV_HOSTS

    if report_format == 'csv results':
        return ReportFormatType.CSV_RESULTS

    if report_format == 'gcr pdf':
        return ReportFormatType.GCR_PDF

    if report_format == 'gsr html':
        return ReportFormatType.GSR_HTML

    if report_format == 'gsr pdf':
        return ReportFormatType.GSR_PDF

    if report_format == 'gxcr pdf':
        return ReportFormatType.GXCR_PDF

    if report_format == 'gxr pdf':
        return ReportFormatType.GXR_PDF

    if report_format == 'itg':
        return ReportFormatType.ITG

    if report_format == 'latex':
        return ReportFormatType.LATEX

    if report_format == 'nbe':
        return ReportFormatType.NBE

    if report_format == 'pdf':
        return ReportFormatType.PDF

    if report_format == 'svg':
        return ReportFormatType.SVG

    if report_format == 'txt':
        return ReportFormatType.TXT

    if report_format == 'verinice ism':
        return ReportFormatType.VERINICE_ISM

    if report_format == 'verinice itg':
        return ReportFormatType.VERINICE_ITG

    if report_format == 'xml':
        return ReportFormatType.XML

    raise InvalidArgument(
        argument='report_format',
        function=get_report_format_id_from_string.__name__,
    )


class ScannerType(Enum):
    """ Enum for scanner type """

    OSP_SCANNER_TYPE = "1"
    OPENVAS_SCANNER_TYPE = "2"
    CVE_SCANNER_TYPE = "3"
    GMP_SCANNER_TYPE = "4"  # formerly slave scanner


def get_scanner_type_from_string(
    scanner_type: Optional[str],
) -> Optional[ScannerType]:
    """Convert a scanner type string to an actual ScannerType instance

    Arguments:
        scanner_type: Scanner type string to convert to a ScannerType
    """
    if not scanner_type:
        return None

    scanner_type = scanner_type.lower()

    if (
        scanner_type == ScannerType.OSP_SCANNER_TYPE.value
        or scanner_type == 'osp'
    ):
        return ScannerType.OSP_SCANNER_TYPE

    if (
        scanner_type == ScannerType.OPENVAS_SCANNER_TYPE.value
        or scanner_type == 'openvas'
    ):
        return ScannerType.OPENVAS_SCANNER_TYPE

    if (
        scanner_type == ScannerType.CVE_SCANNER_TYPE.value
        or scanner_type == 'cve'
    ):
        return ScannerType.CVE_SCANNER_TYPE

    if (
        scanner_type == ScannerType.GMP_SCANNER_TYPE.value
        or scanner_type == 'gmp'
    ):
        return ScannerType.GMP_SCANNER_TYPE

    raise InvalidArgument(
        argument='scanner_type', function=get_scanner_type_from_string.__name__
    )


class SnmpAuthAlgorithm(Enum):
    """ Enum for SNMP auth algorithm """

    SHA1 = 'sha1'
    MD5 = 'md5'


def get_snmp_auth_algorithm_from_string(
    algorithm: Optional[str],
) -> Optional[SnmpAuthAlgorithm]:
    """Convert a SNMP auth algorithm string into a SnmpAuthAlgorithm instance"""
    if not algorithm:
        return None

    try:
        return SnmpAuthAlgorithm[algorithm.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='algorithm',
            function=get_snmp_auth_algorithm_from_string.__name__,
        ) from None


class SnmpPrivacyAlgorithm(Enum):
    """ Enum for SNMP privacy algorithm """

    AES = 'aes'
    DES = 'des'


def get_snmp_privacy_algorithm_from_string(
    algorithm: Optional[str],
) -> Optional[SnmpPrivacyAlgorithm]:
    """Convert a SNMP privacy algorithm string into a SnmpPrivacyAlgorithm
    instance
    """
    if not algorithm:
        return None

    try:
        return SnmpPrivacyAlgorithm[algorithm.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='algorithm',
            function=get_snmp_privacy_algorithm_from_string.__name__,
        ) from None


class SeverityLevel(Enum):
    """ Enum for severity levels """

    HIGH = "High"
    MEDIUM = "Medium"
    LOW = "Low"
    LOG = "Log"
    ALARM = "Alarm"
    DEBUG = "Debug"


def get_severity_level_from_string(
    severity_level: Optional[str],
) -> Optional[SeverityLevel]:
    """ Convert a severity level string into a SeverityLevel instance """
    if not severity_level:
        return None

    try:
        return SeverityLevel[severity_level.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='severity_level',
            function=get_severity_level_from_string.__name__,
        ) from None


class TimeUnit(Enum):
    """ Enum for time units """

    SECOND = "second"
    MINUTE = "minute"
    HOUR = "hour"
    DAY = "day"
    WEEK = "week"
    MONTH = "month"
    YEAR = "year"
    DECADE = "decade"


def get_time_unit_from_string(time_unit: Optional[str]) -> Optional[TimeUnit]:
    """ Convert a time unit string into a TimeUnit instance """
    if not time_unit:
        return None

    try:
        return TimeUnit[time_unit.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='severity_level',
            function=get_severity_level_from_string.__name__,
        ) from None



# -*- coding: utf-8 -*-
# Copyright (C) 2018 - 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=arguments-differ, redefined-builtin, too-many-lines

"""
Module for communication with gvmd in `Greenbone Management Protocol version 8`_

.. _Greenbone Management Protocol version 8:
    https://docs.greenbone.net/API/GMP/gmp-8.0.html
"""
import warnings

from typing import Any, List, Optional, Callable

from gvm.errors import InvalidArgument, InvalidArgumentType, RequiredArgument
from gvm.xml import XmlCommand

from gvm.protocols.base import GvmProtocol
from gvm.connections import GvmConnection
from gvm.protocols.gmpv7.gmpv7 import _to_bool, _add_filter

from . import types
from .types import *  # pylint: disable=unused-wildcard-import, wildcard-import


class GmpV8Mixin(GvmProtocol):
    types = types

    def __init__(
        self,
        connection: GvmConnection,
        *,
        transform: Optional[Callable[[str], Any]] = None
    ):
        super().__init__(connection, transform=transform)

        # Is authenticated on gvmd
        self._authenticated = False

    def create_credential(
        self,
        name: str,
        credential_type: CredentialType,
        *,
        comment: Optional[str] = None,
        allow_insecure: Optional[bool] = None,
        certificate: Optional[str] = None,
        key_phrase: Optional[str] = None,
        private_key: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        auth_algorithm: Optional[SnmpAuthAlgorithm] = None,
        community: Optional[str] = None,
        privacy_algorithm: Optional[SnmpPrivacyAlgorithm] = None,
        privacy_password: Optional[str] = None,
        public_key: Optional[str] = None
    ) -> Any:
        """Create a new credential

        Create a new credential e.g. to be used in the method of an alert.

        Currently the following credential types are supported:

            - Username + Password
            - Username + SSH-Key
            - Client Certificates
            - SNMPv1 or SNMPv2c protocol
            - S/MIME Certificate
            - OpenPGP Key
            - Password only

        Arguments:
            name: Name of the new credential
            credential_type: The credential type.
            comment: Comment for the credential
            allow_insecure: Whether to allow insecure use of the credential
            certificate: Certificate for the credential.
                Required for client-certificate and smime credential types.
            key_phrase: Key passphrase for the private key.
                Used for the username+ssh-key credential type.
            private_key: Private key to use for login. Required
                for usk credential type. Also used for the cc credential type.
                The supported key types (dsa, rsa, ecdsa, ...) and formats (PEM,
                PKC#12, OpenSSL, ...) depend on your installed GnuTLS version.
            login: Username for the credential. Required for username+password,
                username+ssh-key and snmp credential type.
            password: Password for the credential. Used for username+password
                and snmp credential types.
            community: The SNMP community
            auth_algorithm: The SNMP authentication algorithm. Required for snmp
                credential type.
            privacy_algorithm: The SNMP privacy algorithm
            privacy_password: The SNMP privacy password
            public_key: PGP public key in *armor* plain text format. Required
                for pgp credential type.

        Examples:
            Creating a Username + Password credential

            .. code-block:: python

                gmp.create_credential(
                    name='UP Credential',
                    credential_type=CredentialType.USERNAME_PASSWORD,
                    login='foo',
                    password='bar',
                );

            Creating a Username + SSH Key credential

            .. code-block:: python

                with open('path/to/private-ssh-key') as f:
                    key = f.read()

                gmp.create_credential(
                    name='USK Credential',
                    credential_type=CredentialType.USERNAME_SSH_KEY,
                    login='foo',
                    key_phrase='foobar',
                    private_key=key,
                )

            Creating a PGP credential

            .. note::

                A compatible public pgp key file can be exported with GnuPG via
                ::

                    $ gpg --armor --export alice@cyb.org > alice.asc

            .. code-block:: python

                with open('path/to/pgp.key.asc') as f:
                    key = f.read()

                gmp.create_credential(
                    name='PGP Credential',
                    credential_type=CredentialType.PGP_ENCRYPTION_KEY,
                    public_key=key,
                )

            Creating a S/MIME credential

            .. code-block:: python

                with open('path/to/smime-cert') as f:
                    cert = f.read()

                gmp.create_credential(
                    name='SMIME Credential',
                    credential_type=CredentialType.SMIME_CERTIFICATE,
                    certificate=cert,
                )

            Creating a Password-Only credential

            .. code-block:: python

                gmp.create_credential(
                    name='Password-Only Credential',
                    credential_type=CredentialType.PASSWORD_ONLY,
                    password='foo',
                )
        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not name:
            raise RequiredArgument(
                function=self.create_credential.__name__, argument='name'
            )

        if not isinstance(credential_type, self.types.CredentialType):
            raise InvalidArgumentType(
                function=self.create_credential.__name__,
                argument='credential_type',
                arg_type=CredentialType.__name__,
            )

        cmd = XmlCommand("create_credential")
        cmd.add_element("name", name)

        cmd.add_element("type", credential_type.value)

        if comment:
            cmd.add_element("comment", comment)

        if allow_insecure is not None:
            cmd.add_element("allow_insecure", _to_bool(allow_insecure))

        if (
            credential_type == CredentialType.CLIENT_CERTIFICATE
            or credential_type == CredentialType.SMIME_CERTIFICATE
        ):
            if not certificate:
                raise RequiredArgument(
                    function=self.create_credential.__name__,
                    argument='certificate',
                )

            cmd.add_element("certificate", certificate)

        if (
            credential_type == CredentialType.USERNAME_PASSWORD
            or credential_type == CredentialType.USERNAME_SSH_KEY
            or credential_type == CredentialType.SNMP
        ):
            if not login:
                raise RequiredArgument(
                    function=self.create_credential.__name__, argument='login'
                )

            cmd.add_element("login", login)

        if credential_type == CredentialType.PASSWORD_ONLY and not password:
            raise RequiredArgument(
                function=self.create_credential.__name__, argument='password'
            )

        if (
            credential_type == CredentialType.USERNAME_PASSWORD
            or credential_type == CredentialType.SNMP
            or credential_type == CredentialType.PASSWORD_ONLY
        ) and password:
            cmd.add_element("password", password)

        if credential_type == CredentialType.USERNAME_SSH_KEY:
            if not private_key:
                raise RequiredArgument(
                    function=self.create_credential.__name__,
                    argument='private_key',
                )

            _xmlkey = cmd.add_element("key")
            _xmlkey.add_element("private", private_key)

            if key_phrase:
                _xmlkey.add_element("phrase", key_phrase)

        if credential_type == CredentialType.CLIENT_CERTIFICATE and private_key:
            _xmlkey = cmd.add_element("key")
            _xmlkey.add_element("private", private_key)

        if credential_type == CredentialType.SNMP:
            if not isinstance(auth_algorithm, self.types.SnmpAuthAlgorithm):
                raise InvalidArgumentType(
                    function=self.create_credential.__name__,
                    argument='auth_algorithm',
                    arg_type=SnmpAuthAlgorithm.__name__,
                )

            cmd.add_element("auth_algorithm", auth_algorithm.value)

            if community:
                cmd.add_element("community", community)

            if privacy_algorithm is not None or privacy_password:
                _xmlprivacy = cmd.add_element("privacy")

                if privacy_algorithm is not None:
                    if not isinstance(
                        privacy_algorithm, self.types.SnmpPrivacyAlgorithm
                    ):
                        raise InvalidArgumentType(
                            function=self.create_credential.__name__,
                            argument='privacy_algorithm',
                            arg_type=SnmpPrivacyAlgorithm.__name__,
                        )

                    _xmlprivacy.add_element(
                        "algorithm", privacy_algorithm.value
                    )

                if privacy_password:
                    _xmlprivacy.add_element("password", privacy_password)

        if credential_type == CredentialType.PGP_ENCRYPTION_KEY:
            if not public_key:
                raise RequiredArgument(
                    function=self.create_credential.__name__,
                    argument='public_key',
                )

            _xmlkey = cmd.add_element("key")
            _xmlkey.add_element("public", public_key)

        return self._send_xml_command(cmd)

    def modify_credential(
        self,
        credential_id: str,
        *,
        name: Optional[str] = None,
        comment: Optional[str] = None,
        allow_insecure: Optional[bool] = None,
        certificate: Optional[str] = None,
        key_phrase: Optional[str] = None,
        private_key: Optional[str] = None,
        login: Optional[str] = None,
        password: Optional[str] = None,
        auth_algorithm: Optional[SnmpAuthAlgorithm] = None,
        community: Optional[str] = None,
        privacy_algorithm: Optional[SnmpPrivacyAlgorithm] = None,
        privacy_password: Optional[str] = None,
        public_key: Optional[str] = None
    ) -> Any:
        """Modifies an existing credential.

        Arguments:
            credential_id: UUID of the credential
            name: Name of the credential
            comment: Comment for the credential
            allow_insecure: Whether to allow insecure use of the credential
            certificate: Certificate for the credential
            key_phrase: Key passphrase for the private key
            private_key: Private key to use for login
            login: Username for the credential
            password: Password for the credential
            auth_algorithm: The authentication algorithm for SNMP
            community: The SNMP community
            privacy_algorithm: The privacy algorithm for SNMP
            privacy_password: The SNMP privacy password
            public_key: PGP public key in *armor* plain text format

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not credential_id:
            raise RequiredArgument(
                function=self.modify_credential.__name__,
                argument='credential_id',
            )

        cmd = XmlCommand("modify_credential")
        cmd.set_attribute("credential_id", credential_id)

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if allow_insecure is not None:
            cmd.add_element("allow_insecure", _to_bool(allow_insecure))

        if certificate:
            cmd.add_element("certificate", certificate)

        if key_phrase and private_key:
            _xmlkey = cmd.add_element("key")
            _xmlkey.add_element("phrase", key_phrase)
            _xmlkey.add_element("private", private_key)
        elif (not key_phrase and private_key) or (
            key_phrase and not private_key
        ):
            raise RequiredArgument(
                function=self.modify_credential.__name__,
                argument='key_phrase and private_key',
            )

        if login:
            cmd.add_element("login", login)

        if password:
            cmd.add_element("password", password)

        if auth_algorithm:
            if not isinstance(auth_algorithm, self.types.SnmpAuthAlgorithm):
                raise InvalidArgumentType(
                    function=self.modify_credential.__name__,
                    argument='auth_algorithm',
                    arg_type=SnmpAuthAlgorithm.__name__,
                )
            cmd.add_element("auth_algorithm", auth_algorithm.value)

        if community:
            cmd.add_element("community", community)

        if privacy_algorithm is not None or privacy_password is not None:
            _xmlprivacy = cmd.add_element("privacy")

            if privacy_algorithm is not None:
                if not isinstance(
                    privacy_algorithm, self.types.SnmpPrivacyAlgorithm
                ):
                    raise InvalidArgumentType(
                        function=self.modify_credential.__name__,
                        argument='privacy_algorithm',
                        arg_type=SnmpPrivacyAlgorithm.__name__,
                    )

                _xmlprivacy.add_element("algorithm", privacy_algorithm.value)

            if privacy_password is not None:
                _xmlprivacy.add_element("password", privacy_password)

        if public_key:
            _xmlkey = cmd.add_element("key")
            _xmlkey.add_element("public", public_key)

        return self._send_xml_command(cmd)

    def create_tag(
        self,
        name: str,
        resource_type: EntityType,
        *,
        resource_filter: Optional[str] = None,
        resource_ids: Optional[List[str]] = None,
        value: Optional[str] = None,
        comment: Optional[str] = None,
        active: Optional[bool] = None
    ) -> Any:
        """Create a tag.

        Arguments:
            name: Name of the tag. A full tag name consisting of namespace and
                predicate e.g. `foo:bar`.
            resource_type: Entity type the tag is to be attached to.
            resource_filter: Filter term to select resources the tag is to be
                attached to. Only one of resource_filter or resource_ids can be
                provided.
            resource_ids: IDs of the resources the tag is to be attached to.
                Only one of resource_filter or resource_ids can be provided.
            value: Value associated with the tag.
            comment: Comment for the tag.
            active: Whether the tag should be active.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not name:
            raise RequiredArgument(
                function=self.create_tag.__name__, argument='name'
            )

        if resource_filter and resource_ids:
            raise InvalidArgument(
                "create_tag accepts either resource_filter or resource_ids "
                "argument",
                function=self.create_tag.__name__,
            )

        if not resource_type:
            raise RequiredArgument(
                function=self.create_tag.__name__, argument='resource_type'
            )

        if not isinstance(resource_type, self.types.EntityType):
            raise InvalidArgumentType(
                function=self.create_tag.__name__,
                argument='resource_type',
                arg_type=EntityType.__name__,
            )

        cmd = XmlCommand('create_tag')
        cmd.add_element('name', name)

        _xmlresources = cmd.add_element("resources")
        if resource_filter is not None:
            _xmlresources.set_attribute("filter", resource_filter)

        for resource_id in resource_ids or []:
            _xmlresources.add_element(
                "resource", attrs={"id": str(resource_id)}
            )

        _xmlresources.add_element("type", resource_type.value)

        if comment:
            cmd.add_element("comment", comment)

        if value:
            cmd.add_element("value", value)

        if active is not None:
            if active:
                cmd.add_element("active", "1")
            else:
                cmd.add_element("active", "0")

        return self._send_xml_command(cmd)

    def modify_tag(
        self,
        tag_id: str,
        *,
        comment: Optional[str] = None,
        name: Optional[str] = None,
        value=None,
        active=None,
        resource_action: Optional[str] = None,
        resource_type: Optional[EntityType] = None,
        resource_filter: Optional[str] = None,
        resource_ids: Optional[List[str]] = None
    ) -> Any:
        """Modifies an existing tag.

        Arguments:
            tag_id: UUID of the tag.
            comment: Comment to add to the tag.
            name: Name of the tag.
            value: Value of the tag.
            active: Whether the tag is active.
            resource_action: Whether to add or remove resources instead of
                overwriting. One of '', 'add', 'set' or 'remove'.
            resource_type: Type of the resources to which to attach the tag.
                Required if resource_filter is set.
            resource_filter: Filter term to select resources the tag is to be
                attached to.
            resource_ids: IDs of the resources to which to attach the tag.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not tag_id:
            raise RequiredArgument(
                function=self.modify_tag.__name__, argument='tag_id'
            )

        cmd = XmlCommand("modify_tag")
        cmd.set_attribute("tag_id", str(tag_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if value:
            cmd.add_element("value", value)

        if active is not None:
            cmd.add_element("active", _to_bool(active))

        if resource_action or resource_filter or resource_ids or resource_type:
            if resource_filter and not resource_type:
                raise RequiredArgument(
                    function=self.modify_tag.__name__, argument='resource_type'
                )

            _xmlresources = cmd.add_element("resources")
            if resource_action is not None:
                _xmlresources.set_attribute("action", resource_action)

            if resource_filter is not None:
                _xmlresources.set_attribute("filter", resource_filter)

            for resource_id in resource_ids or []:
                _xmlresources.add_element(
                    "resource", attrs={"id": str(resource_id)}
                )

            if resource_type is not None:
                if not isinstance(resource_type, self.types.EntityType):
                    raise InvalidArgumentType(
                        function=self.modify_tag.__name__,
                        argument="resource_type",
                        arg_type=EntityType.__name__,
                    )
                _xmlresources.add_element("type", resource_type.value)

        return self._send_xml_command(cmd)

    def create_target(
        self,
        name: str,
        *,
        make_unique: Optional[bool] = None,
        asset_hosts_filter: Optional[str] = None,
        hosts: Optional[List[str]] = None,
        comment: Optional[str] = None,
        exclude_hosts: Optional[List[str]] = None,
        ssh_credential_id: Optional[str] = None,
        ssh_credential_port: Optional[int] = None,
        smb_credential_id: Optional[str] = None,
        esxi_credential_id: Optional[str] = None,
        snmp_credential_id: Optional[str] = None,
        alive_test: Optional[AliveTest] = None,
        reverse_lookup_only: Optional[bool] = None,
        reverse_lookup_unify: Optional[bool] = None,
        port_range: Optional[str] = None,
        port_list_id: Optional[str] = None
    ) -> Any:
        """Create a new target

        Arguments:
            name: Name of the target
            make_unique: Deprecated. Will be ignored.
            asset_hosts_filter: Filter to select target host from assets hosts
            hosts: List of hosts addresses to scan
            exclude_hosts: List of hosts addresses to exclude from scan
            comment: Comment for the target
            ssh_credential_id: UUID of a ssh credential to use on target
            ssh_credential_port: The port to use for ssh credential
            smb_credential_id: UUID of a smb credential to use on target
            snmp_credential_id: UUID of a snmp credential to use on target
            esxi_credential_id: UUID of a esxi credential to use on target
            alive_test: Which alive test to use
            reverse_lookup_only: Whether to scan only hosts that have names
            reverse_lookup_unify: Whether to scan only one IP when multiple IPs
                have the same name.
            port_range: Port range for the target
            port_list_id: UUID of the port list to use on target

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if make_unique is not None:
            warnings.warn(
                'create_target make_unique argument is deprecated '
                'and will be ignored.',
                DeprecationWarning,
            )

        return super().create_target(
            name,
            asset_hosts_filter=asset_hosts_filter,
            hosts=hosts,
            comment=comment,
            exclude_hosts=exclude_hosts,
            ssh_credential_id=ssh_credential_id,
            ssh_credential_port=ssh_credential_port,
            smb_credential_id=smb_credential_id,
            esxi_credential_id=esxi_credential_id,
            snmp_credential_id=snmp_credential_id,
            alive_test=alive_test,
            reverse_lookup_only=reverse_lookup_only,
            reverse_lookup_unify=reverse_lookup_unify,
            port_range=port_range,
            port_list_id=port_list_id,
        )

    def clone_ticket(self, ticket_id: str) -> Any:
        """Clone an existing ticket

        Arguments:
            ticket_id: UUID of an existing ticket to clone from

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not ticket_id:
            raise RequiredArgument(
                function=self.clone_ticket.__name__, argument='ticket_id'
            )

        cmd = XmlCommand("create_ticket")

        _copy = cmd.add_element("copy", ticket_id)

        return self._send_xml_command(cmd)

    def create_ticket(
        self,
        *,
        result_id: str,
        assigned_to_user_id: str,
        note: str,
        comment: Optional[str] = None
    ) -> Any:
        """Create a new ticket

        Arguments:
            result_id: UUID of the result the ticket applies to
            assigned_to_user_id: UUID of a user the ticket should be assigned to
            note: A note about opening the ticket
            comment: Comment for the ticket

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not result_id:
            raise RequiredArgument(
                function=self.create_ticket.__name__, argument='result_id'
            )

        if not assigned_to_user_id:
            raise RequiredArgument(
                function=self.create_ticket.__name__,
                argument='assigned_to_user_id',
            )

        if not note:
            raise RequiredArgument(
                function=self.create_ticket.__name__, argument='note'
            )

        cmd = XmlCommand("create_ticket")

        _result = cmd.add_element("result")
        _result.set_attribute("id", result_id)

        _assigned = cmd.add_element("assigned_to")
        _user = _assigned.add_element("user")
        _user.set_attribute("id", assigned_to_user_id)

        _note = cmd.add_element("open_note", note)

        if comment:
            cmd.add_element("comment", comment)

        return self._send_xml_command(cmd)

    def delete_ticket(
        self, ticket_id: str, *, ultimate: Optional[bool] = False
    ):
        """Deletes an existing ticket

        Arguments:
            ticket_id: UUID of the ticket to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not ticket_id:
            raise RequiredArgument(
                function=self.delete_ticket.__name__, argument='ticket_id'
            )

        cmd = XmlCommand("delete_ticket")
        cmd.set_attribute("ticket_id", ticket_id)
        cmd.set_attribute("ultimate", _to_bool(ultimate))

        return self._send_xml_command(cmd)

    def get_tickets(
        self,
        *,
        trash: Optional[bool] = None,
        filter: Optional[str] = None,
        filter_id: Optional[str] = None
    ) -> Any:
        """Request a list of tickets

        Arguments:
            filter: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: True to request the tickets in the trashcan

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        cmd = XmlCommand("get_tickets")

        _add_filter(cmd, filter, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", _to_bool(trash))

        return self._send_xml_command(cmd)

    def get_ticket(self, ticket_id: str) -> Any:
        """Request a single ticket

        Arguments:
            ticket_id: UUID of an existing ticket

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not ticket_id:
            raise RequiredArgument(
                function=self.get_ticket.__name__, argument='ticket_id'
            )

        cmd = XmlCommand("get_tickets")
        cmd.set_attribute("ticket_id", ticket_id)
        return self._send_xml_command(cmd)

    def get_vulnerabilities(
        self, *, filter: Optional[str] = None, filter_id: Optional[str] = None
    ) -> Any:
        """Request a list of vulnerabilities

        Arguments:
            filter: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        cmd = XmlCommand("get_vulns")

        _add_filter(cmd, filter, filter_id)

        return self._send_xml_command(cmd)

    def get_vulnerability(self, vulnerability_id: str) -> Any:
        """Request a single vulnerability

        Arguments:
            vulnerability_id: ID of an existing vulnerability

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not vulnerability_id:
            raise RequiredArgument(
                function=self.get_vulnerability.__name__,
                argument='vulnerability_id',
            )

        cmd = XmlCommand("get_vulns")
        cmd.set_attribute("vuln_id", vulnerability_id)
        return self._send_xml_command(cmd)

    def modify_ticket(
        self,
        ticket_id: str,
        *,
        status: Optional[TicketStatus] = None,
        note: Optional[str] = None,
        assigned_to_user_id: Optional[str] = None,
        comment: Optional[str] = None
    ) -> Any:
        """Modify a single ticket

        Arguments:
            ticket_id: UUID of an existing ticket
            status: New status for the ticket
            note: Note for the status change. Required if status is set.
            assigned_to_user_id: UUID of the user the ticket should be assigned
                to
            comment: Comment for the ticket

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not ticket_id:
            raise RequiredArgument(
                function=self.modify_ticket.__name__, argument='ticket_id'
            )

        if status and not note:
            raise RequiredArgument(
                function=self.modify_ticket.__name__, argument='note'
            )

        if note and not status:
            raise RequiredArgument(
                function=self.modify_ticket.__name__, argument='status'
            )

        cmd = XmlCommand("modify_ticket")
        cmd.set_attribute("ticket_id", ticket_id)

        if assigned_to_user_id:
            _assigned = cmd.add_element("assigned_to")
            _user = _assigned.add_element("user")
            _user.set_attribute("id", assigned_to_user_id)

        if status:
            if not isinstance(status, self.types.TicketStatus):
                raise InvalidArgumentType(
                    function=self.modify_ticket.__name__,
                    argument='status',
                    arg_type=TicketStatus.__name__,
                )

            cmd.add_element('status', status.value)
            cmd.add_element('{}_note'.format(status.name.lower()), note)

        if comment:
            cmd.add_element("comment", comment)

        return self._send_xml_command(cmd)

    def create_filter(
        self,
        name: str,
        *,
        filter_type: Optional[FilterType] = None,
        comment: Optional[str] = None,
        term: Optional[str] = None
    ) -> Any:
        """Create a new filter

        Arguments:
            name: Name of the new filter
            filter_type: Filter for entity type
            comment: Comment for the filter
            term: Filter term e.g. 'name=foo'

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not name:
            raise RequiredArgument(
                function=self.create_filter.__name__, argument="name"
            )

        cmd = XmlCommand("create_filter")
        _xmlname = cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if term:
            cmd.add_element("term", term)

        if filter_type:
            if not isinstance(filter_type, self.types.FilterType):
                raise InvalidArgumentType(
                    function=self.create_filter.__name__,
                    argument="filter_type",
                    arg_type=self.types.FilterType.__name__,
                )

            cmd.add_element("type", filter_type.value)

        return self._send_xml_command(cmd)

    def modify_filter(
        self,
        filter_id: str,
        *,
        comment: Optional[str] = None,
        name: Optional[str] = None,
        term: Optional[str] = None,
        filter_type: Optional[FilterType] = None
    ) -> Any:
        """Modifies an existing filter.

        Arguments:
            filter_id: UUID of the filter to be modified
            comment: Comment on filter.
            name: Name of filter.
            term: Filter term.
            filter_type: Resource type filter applies to.

        Returns:
            The response. See :py:meth:`send_command` for details.
        """
        if not filter_id:
            raise RequiredArgument(
                function=self.modify_filter.__name__, argument='filter_id'
            )

        cmd = XmlCommand("modify_filter")
        cmd.set_attribute("filter_id", filter_id)

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if term:
            cmd.add_element("term", term)

        if filter_type:
            if not isinstance(filter_type, self.types.FilterType):
                raise InvalidArgumentType(
                    function=self.modify_filter.__name__,
                    argument='filter_type',
                    arg_type=FilterType.__name__,
                )
            cmd.add_element("type", filter_type.value)

        return self._send_xml_command(cmd)

    def create_schedule(
        self,
        name: str,
        icalendar: str,
        timezone: str,
        *,
        comment: Optional[str] = None
    ) -> Any:
        """Create a new schedule based in `iCalendar`_ data.

        Example:
            Requires https://pypi.org/project/icalendar/

            .. code-block:: python

                import pytz

                from datetime import datetime

                from icalendar import Calendar, Event

                cal = Calendar()

                cal.add('prodid', '-//Foo Bar//')
                cal.add('version', '2.0')

                event = Event()
                event.add('dtstamp', datetime.now(tz=pytz.UTC))
                event.add('dtstart', datetime(2020, 1, 1, tzinfo=pytz.utc))

                cal.add_component(event)

                gmp.create_schedule(
                    name="My Schedule",
                    icalendar=cal.to_ical(),
                    timezone='UTC'
                )


        Arguments:
            name: Name of the new schedule
            icalendar: `iCalendar`_ (RFC 5545) based data.
            timezone: Timezone to use for the icalender events e.g
                Europe/Berlin. If the datetime values in the icalendar data are
                missing timezone information this timezone gets applied.
                Otherwise the datetime values from the icalendar data are
                displayed in this timezone
            comment: Comment on schedule.

        Returns:
            The response. See :py:meth:`send_command` for details.

        .. _iCalendar:
            https://tools.ietf.org/html/rfc5545
        """
        if not name:
            raise RequiredArgument(
                function=self.create_schedule.__name__, argument='name'
            )
        if not icalendar:
            raise RequiredArgument(
                function=self.create_schedule.__name__, argument='icalendar'
            )
        if not timezone:
            raise RequiredArgument(
                function=self.create_schedule.__name__, argument='timezone'
            )

        cmd = XmlCommand("create_schedule")

        cmd.add_element("name", name)
        cmd.add_element("icalendar", icalendar)
        cmd.add_element("timezone", timezone)

        if comment:
            cmd.add_element("comment", comment)

        return self._send_xml_command(cmd)

    def modify_schedule(
        self,
        schedule_id: str,
        *,
        name: Optional[str] = None,
        icalendar: Optional[str] = None,
        timezone: Optional[str] = None,
        comment: Optional[str] = None
    ) -> Any:
        """Modifies an existing schedule

        Arguments:
            schedule_id: UUID of the schedule to be modified
            name: Name of the schedule
            icalendar: `iCalendar`_ (RFC 5545) based data.
            timezone: Timezone to use for the icalender events e.g
                Europe/Berlin. If the datetime values in the icalendar data are
                missing timezone information this timezone gets applied.
                Otherwise the datetime values from the icalendar data are
                displayed in this timezone
            commenhedule.

        Returns:
            The response. See :py:meth:`send_command` for details.

        .. _iCalendar:
            https://tools.ietf.org/html/rfc5545
        """
        if not schedule_id:
            raise RequiredArgument(
                function=self.modify_schedule.__name__, argument='schedule_id'
            )

        cmd = XmlCommand("modify_schedule")
        cmd.set_attribute("schedule_id", schedule_id)

        if name:
            cmd.add_element("name", name)

        if icalendar:
            cmd.add_element("icalendar", icalendar)

        if timezone:
            cmd.add_element("timezone", timezone)

        if comment:
            cmd.add_element("comment", comment)

        return self._send_xml_command(cmd)



# -*- coding: utf-8 -*-
# Copyright (C) 2020 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from enum import Enum

from typing import Optional

from gvm.errors import InvalidArgument


from gvm.protocols.gmpv208.types import (
    AlertCondition,
    AlertEvent,
    AlertMethod,
    AliveTest,
    AssetType,
    CredentialFormat,
    CredentialType,
    EntityType,
    FeedType,
    FilterType,
    HostsOrdering,
    InfoType,
    PermissionSubjectType,
    PortRangeType,
    ScannerType,
    SnmpAuthAlgorithm,
    SnmpPrivacyAlgorithm,
    TicketStatus,
    TimeUnit,
    get_alert_condition_from_string,
    get_alert_event_from_string,
    get_alert_method_from_string,
    get_alive_test_from_string,
    get_asset_type_from_string,
    get_credential_format_from_string,
    get_credential_type_from_string,
    get_entity_type_from_string,
    get_feed_type_from_string,
    get_filter_type_from_string,
    get_hosts_ordering_from_string,
    get_info_type_from_string,
    get_permission_subject_type_from_string,
    get_port_range_type_from_string,
    get_scanner_type_from_string,
    get_snmp_auth_algorithm_from_string,
    get_snmp_privacy_algorithm_from_string,
    get_ticket_status_from_string,
    get_time_unit_from_string,
)


__all__ = [
    "AlertCondition",
    "AlertEvent",
    "AlertMethod",
    "AliveTest",
    "AssetType",
    "CredentialFormat",
    "CredentialType",
    "EntityType",
    "FeedType",
    "FilterType",
    "HostsOrdering",
    "InfoType",
    "PermissionSubjectType",
    "PortRangeType",
    "ScannerType",
    "SeverityLevel",
    "SnmpAuthAlgorithm",
    "SnmpPrivacyAlgorithm",
    "TicketStatus",
    "TimeUnit",
    "get_alert_condition_from_string",
    "get_alert_event_from_string",
    "get_alert_method_from_string",
    "get_alive_test_from_string",
    "get_asset_type_from_string",
    "get_credential_format_from_string",
    "get_credential_type_from_string",
    "get_entity_type_from_string",
    "get_feed_type_from_string",
    "get_filter_type_from_string",
    "get_hosts_ordering_from_string",
    "get_info_type_from_string",
    "get_permission_subject_type_from_string",
    "get_port_range_type_from_string",
    "get_scanner_type_from_string",
    "get_severity_level_from_string",
    "get_snmp_auth_algorithm_from_string",
    "get_snmp_privacy_algorithm_from_string",
    "get_ticket_status_from_string",
    "get_time_unit_from_string",
]


class SeverityLevel(Enum):
    """ Enum for severity levels """

    HIGH = "High"
    MEDIUM = "Medium"
    LOW = "Low"
    LOG = "Log"
    ALARM = "Alarm"


def get_severity_level_from_string(
    severity_level: Optional[str],
) -> Optional[SeverityLevel]:
    """ Convert a severity level string into a SeverityLevel instance """
    if not severity_level:
        return None

    try:
        return SeverityLevel[severity_level.upper()]
    except KeyError:
        raise InvalidArgument(
            argument='severity_level',
            function=get_severity_level_from_string.__name__,
        ) from None



# -*- coding utf-8 -*-
# Copyright (C) 2020 Greenbone Networks GmbH
#
# SPDX-License-Identifier GPL-3.0-or-later
#
# This program is free software you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http//www.gnu.org/licenses/>.

# pylint: disable=no-member

from .test_clone_audit import GmpCloneAuditTestCase
from .test_clone_config import GmpCloneConfigTestCase
from .test_clone_policy import GmpClonePolicyTestCase
from .test_clone_task import GmpCloneTaskTestCase
from .test_clone_tls_certificate import GmpCloneTLSCertificateTestCase
from .test_create_audit import GmpCreateAuditCommandTestCase
from .test_create_alert import GmpCreateAlertTestCase
from .test_create_config import GmpCreateConfigTestCase
from .test_create_config_from_osp_scanner import (
    GmpCreateConfigFromOSPScannerTestCase,
)
from .test_create_credential import GmpCreateCredentialTestCase
from .test_create_filter import GmpCreateFilterTestCase
from .test_create_permission import GmpCreatePermissionTestCase
from .test_create_policy import GmpCreatePolicyTestCase
from .test_create_scanner import GmpCreateScannerTestCase
from .test_create_tag import GmpCreateTagTestCase
from .test_create_task import GmpCreateTaskCommandTestCase
from .test_create_tls_certificate import GmpCreateTLSCertificateTestCase
from .test_delete_audit import GmpDeleteAuditTestCase
from .test_delete_config import GmpDeleteConfigTestCase
from .test_delete_policy import GmpDeletePolicyTestCase
from .test_delete_task import GmpDeleteTaskTestCase
from .test_delete_tls_certificate import GmpDeleteTLSCertificateTestCase
from .test_get_aggregates import GmpGetAggregatesTestCase
from .test_get_audit import GmpGetAuditTestCase
from .test_get_audits import GmpGetAuditsTestCase
from .test_get_config import GmpGetConfigTestCase
from .test_get_configs import GmpGetConfigsTestCase
from .test_get_policies import GmpGetPoliciesTestCase
from .test_get_policy import GmpGetPolicyTestCase
from .test_get_task import GmpGetTaskTestCase
from .test_get_tasks import GmpGetTasksTestCase
from .test_get_tls_certificate import GmpGetTlsCertificateTestCase
from .test_get_tls_certificates import GmpGetTLSCertificatesTestCase
from .test_modify_credential import GmpModifyCredentialTestCase
from .test_modify_alert import GmpModifyAlertTestCase
from .test_modify_audit import GmpModifyAuditCommandTestCase
from .test_modify_filter import GmpModifyFilterTestCase
from .test_modify_permission import GmpModifyPermissionTestCase
from .test_modify_policy_set_comment import GmpModifyPolicySetCommentTestCase
from .test_modify_policy_set_family_selection import (
    GmpModifyPolicySetFamilySelectionTestCase,
)
from .test_modify_policy_set_name import GmpModifyPolicySetNameTestCase
from .test_modify_policy_set_nvt_preference import (
    GmpModifyPolicySetNvtPreferenceTestCase,
)
from .test_modify_policy_set_nvt_selection import (
    GmpModifyPolicySetNvtSelectionTestCase,
)
from .test_modify_policy_set_scanner_preference import (
    GmpModifyPolicySetScannerPreferenceTestCase,
)
from .test_modify_report_format import GmpModifyReportFormatTestCase
from .test_modify_scanner import GmpModifyScannerTestCase
from .test_modify_tag import GmpModifyTagTestCase
from .test_modify_ticket import GmpModifyTicketTestCase
from .test_modify_tls_certificate import GmpModifyTLSCertificateTestCase



# -*- coding: utf-8 -*-
# Copyright (C) 2019 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from . import Gmpv9TestCase
from ..gmpv8.testcmds import *  # pylint: disable=unused-wildcard-import,wildcard-import


class Gmpv9CloneTicketTestCase(GmpCloneTicketTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateCredentialTestCase(GmpCreateCredentialTestCase, Gmpv9TestCase):

    pass


class Gmpv9CreateScheduleTestCase(GmpCreateScheduleTestCase, Gmpv9TestCase):
    pass


class Gmpv9CreateTargetCommandTestCase(
    GmpCreateTargetCommandTestCase, Gmpv9TestCase
):
    pass


class Gmpv9CreateTicketTestCase(GmpCreateTicketTestCase, Gmpv9TestCase):
    pass


class Gmpv9DeleteTicketTestCase(GmpDeleteTicketTestCase, Gmpv9TestCase):

    pass


class Gmpv9GetTicketTestCase(GmpGetTicketTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetTargetsTestCase(GmpGetTargetsTestCase, Gmpv9TestCase):
    pass


class Gmpv9GetVulnerabilitiesTestCase(
    GmpGetVulnerabilitiesTestCase, Gmpv9TestCase
):
    pass


class Gmpv9GetVulnerabilityTestCase(GmpGetVulnerabilityTestCase, Gmpv9TestCase):

    pass


class Gmpv9ModifyTestCase(GmpModifyTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyScheduleTestCase(GmpModifyScheduleTestCase, Gmpv9TestCase):
    pass


class Gmpv9ModifyTicketTestCase(GmpModifyTicketTestCase, Gmpv9TestCase):
    pass



# -*- coding: utf-8 -*-
# Copyright (C) 2020 Greenbone Networks GmbH
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from gvm.errors import RequiredArgument


class GmpModifyPolicySetNameTestCase:
    def test_modify_policy_set_name(self):
        self.gmp.modify_policy_set_name('c1', 'foo')

        self.connection.send.has_been_called_with(
            '<modify_config config_id="c1">'
            '<name>foo</name>'
            '</modify_config>'
        )

    def test_modify_policy_set_name_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_name(policy_id=None, name='name')

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_name('', name='name')

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_name(policy_id='', name='name')

    def test_modify_policy_set_name_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_name(policy_id='c', name='')

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_name(policy_id='c', name=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_policy_set_name('c', '')


if __name__ == '__main__':
    unittest.main()
